<p>
    <strong>Par: $authors</strong>
</p>

$body

<hr>
<p>
    Vous recevez cet article car vous êtes abonné à $feed_title,
    <a href="mailto:${BOT_EMAIL}?subject=${UNSUBSCRIBE_KEYWORD}%20${feed_url}">se désabonner</a>
</p>