<p>
    Vous êtes abonné au flux $url, les articles devraient arriver prochainement.
    <br> Pour les filter dans votre client mail, vous pouvez utiliser le header X-Flumel-Title = "${title}"
</p>

<p>Pour ne plus recevoir d’articles de ce flux:
    <a href="mailto:${BOT_EMAIL}?subject=${UNSUBSCRIBE_KEYWORD}%20${url}">se désabonner</a>
</p>